/* krb5dissect.c --- Dissect Kerberos credential files.
 * Copyright (C) 2007 Simon Josefsson.
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This file is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this file; see the file COPYING.  If not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

/* Autoconf definitions. */
#include <config.h>

/* Standard headers. */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

/* Gnulib headers. */
#include "progname.h"
#include "version-etc.h"
#include "read-file.h"
#include "error.h"

/* Our headers. */
#include "ccache.h"
#include "krb5dissect_cmd.h"

const char version_etc_copyright[] = "Copyright %s %d Simon Josefsson.";

struct gengetopt_args_info args_info;

void
read_ccache (const char *filename)
{
  size_t len;
  char *data;
  struct ccache ccache;
  struct ccache_credential cred;
  int rc;
  size_t i = 0;

  if (strcmp (filename, "-") == 0)
    data = fread_file (stdin, &len);
  else
    data = read_binary_file (filename, &len);
  if (!data)
    error (EXIT_FAILURE, errno, "%s", filename);

  rc = ccache_parse (data, len, &ccache);
  if (rc < 0)
    error (EXIT_FAILURE, 0, "%s: cannot parse ccache file", filename);

  ccache_print (&ccache);

  while (ccache.credentialslen)
    {
      size_t n;

      rc = ccache_parse_credential (ccache.credentials,
				    ccache.credentialslen, &cred, &n);
      if (rc < 0)
	error (EXIT_FAILURE, 0, "Cannot parse credential %d", i);

      printf ("\nCredential %d:\n", i++);

      ccache_print_credential (&cred);

      ccache.credentials += n;
      ccache.credentialslen -= n;
    }
}

int
main (int argc, char *argv[])
{
  set_program_name (argv[0]);

  if (cmdline_parser (argc, argv, &args_info) != 0)
    return EXIT_FAILURE;

  if (!args_info.quiet_given)
    {
      version_etc (stdout, program_name, PACKAGE, VERSION,
		   "Simon Josefsson", (char*) NULL);
      puts("");
    }

  if (argc < 2)
    {
      cmdline_parser_print_help ();
      return EXIT_FAILURE;
    }

  read_ccache (args_info.inputs[0]);

  return 0;
}
